import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { $ } from "@typespec/compiler/experimental/typekit";
import { FunctionDeclaration } from "@typespec/emitter-framework/typescript";
import { reportDiagnostic } from "../../lib.js";
import { buildClientParameters } from "../../utils/parameters.js";
import { httpRuntimeTemplateLib } from "../external-packages/ts-http-runtime.js";
import { addClientTestOptions } from "../testing/client-options.js";
import { getClientcontextDeclarationRef } from "./client-context-declaration.js";
import { ParametrizedEndpoint } from "./parametrized-endpoint.js";
export function getClientContextFactoryRef(client) {
  return ay.refkey(client, "contextFactory");
}
export function ClientContextFactoryDeclaration(props) {
  const ref = getClientContextFactoryRef(props.client);
  const contextDeclarationRef = getClientcontextDeclarationRef(props.client);
  const namePolicy = ts.useTSNamePolicy();
  const factoryFunctionName = namePolicy.getName(`create_${props.client.name}Context`, "function");
  const clientConstructor = $.client.getConstructor(props.client);
  const parameters = buildClientParameters(props.client);
  const urlTemplate = $.client.getUrlTemplate(props.client);
  const endpointRef = ay.refkey();
  const resolvedEndpoint = _$createComponent(ParametrizedEndpoint, {
    refkey: endpointRef,
    get template() {
      return urlTemplate.url;
    },
    get params() {
      return urlTemplate.parameters;
    }
  });
  let credentialsRef;
  if (clientConstructor.parameters.properties.has("credential")) {
    credentialsRef = "credential";
  }

  // Filter out optional parameters, they will be passed as options
  const args = _$createComponent(ClientFactoryArguments, {
    get client() {
      return props.client;
    },
    endpointRef: endpointRef,
    credentialsRef: credentialsRef
  });
  return _$createComponent(FunctionDeclaration, {
    "export": true,
    name: factoryFunctionName,
    type: clientConstructor,
    returnType: contextDeclarationRef,
    refkey: ref,
    parametersMode: "replace",
    parameters: parameters,
    get children() {
      return [resolvedEndpoint, "return ", _$createComponent(ts.FunctionCallExpression, {
        get target() {
          return httpRuntimeTemplateLib.getClient;
        },
        args: [args]
      })];
    }
  });
}
function ClientFactoryArguments(props) {
  const params = [[_$memo(() => props.endpointRef), ","]];
  if (props.credentialsRef) {
    params.push([_$memo(() => props.credentialsRef), ","]);
  }
  return [...params, _$createComponent(ClientOptionsExpression, {
    get children() {
      return _$createComponent(CredentialOptions, {
        get client() {
          return props.client;
        }
      });
    }
  })];
}
function CredentialOptions(props) {
  const clientCredential = $.client.getAuth(props.client);
  if (clientCredential.schemes.length === 0) {
    return null;
  }
  if (clientCredential.schemes.length !== 1) {
    reportDiagnostic($.program, {
      code: "multiple-auth-schemes-not-yet-supported",
      target: props.client.type
    });
  }
  const scheme = clientCredential.schemes[0];
  switch (scheme.type) {
    case "http":
      // Todo: handle scopes?
      return _$createComponent(ts.ObjectProperty, {
        name: "credentials",
        get children() {
          return _$createComponent(ts.ObjectExpression, {
            get children() {
              return _$createComponent(ts.ObjectProperty, {
                name: "apiKeyHeaderName",
                jsValue: "Authorization"
              });
            }
          });
        }
      });
    case "apiKey":
      if (scheme.in !== "header") {
        reportDiagnostic($.program, {
          code: "non-model-parts",
          target: props.client.service
        });
        return null;
      }
      return _$createComponent(ts.ObjectProperty, {
        name: "credentials",
        get children() {
          return _$createComponent(ts.ObjectExpression, {
            get children() {
              return _$createComponent(ts.ObjectProperty, {
                name: "apiKeyHeaderName",
                get jsValue() {
                  return scheme.name;
                }
              });
            }
          });
        }
      });
    default:
      return null;
  }
}
function ClientOptionsExpression(props) {
  const options = ["...options"];

  // Conditionally add test options
  // based on the environment variable TYPESPEC_JS_EMITTER_TESTING
  addClientTestOptions(options);
  const children = Array.isArray(props.children) ? props.children : [props.children];
  if (children.length) {
    const children = Array.isArray(props.children) ? props.children : [props.children];
    options.push(...children);
  }
  return _$createComponent(ts.ObjectExpression, {
    get children() {
      return _$createComponent(ay.For, {
        each: options,
        joiner: ",",
        line: true,
        children: child => child
      });
    }
  });
}
//# sourceMappingURL=client-context-factory.js.map